@@ -10,7 +10,7 @@ use Monitoring::Livestatus::INET;
 use Monitoring::Livestatus::UNIX;
 use Monitoring::Livestatus::MULTI;
 
-our $VERSION = '0.41_1';
+our $VERSION = '0.30';
 
 
 =head1 NAME
@@ -95,23 +95,12 @@ currently only querys without Columns: Header will result in a warning
 
 =item timeout
 
-set a general timeout. Used for connect and querys, no default
-
-=item query_timeout
-
-set a query timeout. Used for retrieving querys, Default 60sec
-
-=item connect_timeout
-
-set a connect timeout. Used for initial connections, default 5sec
+set a general timeout. Used for connect and querys, Default 10sec
 
 =item use_threads
 
 only used with multiple backend connections.
-Default is to don't threads where available. As threads in perl
-are causing problems with tied resultset and using more memory.
-Querys are usually faster without threads, except for very slow backends
-connections.
+Default is to use threads where available.
 
 =back
 
@@ -138,14 +127,10 @@ sub new {
       "keepalive"                 => 0,       # enable keepalive?
       "errors_are_fatal"          => 1,       # die on errors
       "backend"                   => undef,   # should be keept undef, used internally
-      "timeout"                   => undef,   # timeout for tcp connections
-      "query_timeout"             => 60,      # query timeout for tcp connections
-      "connect_timeout"           => 5,       # connect timeout for tcp connections
-      "timeout"                   => undef,   # timeout for tcp connections
+      "timeout"                   => 10,      # timeout for tcp connections
       "use_threads"               => undef,   # use threads, default is to use threads where available
       "warnings"                  => 1,       # show warnings, for example on querys without Column: Header
       "logger"                    => undef,   # logger object used for statistical informations and errors / warnings
-      "deepcopy"                  => undef,   # copy result set to avoid errors with tied structures
     };
 
     for my $opt_key (keys %options) {
@@ -157,12 +142,6 @@ sub new {
         }
     }
 
-    # setting a general timeout?
-    if(defined $self->{'timeout'}) {
-        $self->{'query_timeout'}   = $self->{'timeout'};
-        $self->{'connect_timeout'} = $self->{'timeout'};
-    }
-
     bless $self, $class;
 
     # set our peer(s) from the options
@@ -199,9 +178,7 @@ sub new {
         $self->{'peer'} = $self->{'CONNECTOR'}->{'peer'};
     }
 
-    if(defined $self->{'logger'} and (!defined $self->{'backend'} or $self->{'backend'} ne 'Monitoring::Livestatus::MULTI')) {
-        $self->{'logger'}->debug('initialized Monitoring::Livestatus ('.$self->peer_name.')');
-    }
+    $self->{'logger'}->debug('initialized Monitoring::Livestatus ('.$self->peer_name.')') if defined $self->{'logger'};
 
     return $self;
 }
@@ -276,7 +253,7 @@ sub selectall_arrayref {
     my $result;
 
     # make opt hash keys lowercase
-    $opt = $self->_lowercase_and_verify_options($opt);
+    %{$opt} = map { lc $_ => $opt->{$_} } keys %{$opt};
 
     if(defined $self->{'logger'}) {
         my $d = Data::Dumper->new([$opt]);
@@ -289,7 +266,11 @@ sub selectall_arrayref {
         $self->{'logger'}->debug('selectall_arrayref("'.$cleanstatement.'", '.$optstring.', '.$limit.')')
     }
 
-    $result = $self->_send($statement, $opt);
+    if(defined $opt->{'addpeer'} and $opt->{'addpeer'}) {
+        $result = $self->_send($statement, 1);
+    } else {
+        $result = $self->_send($statement);
+    }
 
     if(!defined $result) {
         return unless $self->{'errors_are_fatal'};
@@ -343,9 +324,7 @@ sub selectall_hashref {
     my $key_field = shift;
     my $opt       = shift;
 
-    $opt = $self->_lowercase_and_verify_options($opt);
-
-    $opt->{'slice'} = 1;
+    $opt->{'Slice'} = 1 unless defined $opt->{'Slice'};
 
     croak("key is required for selectall_hashref") if !defined $key_field;
 
@@ -416,7 +395,7 @@ sub selectcol_arrayref {
     my $opt       = shift;
 
     # make opt hash keys lowercase
-    $opt = $self->_lowercase_and_verify_options($opt);
+    %{$opt} = map { lc $_ => $opt->{$_} } keys %{$opt};
 
     # if now colums are set, use just the first one
     if(!defined $opt->{'columns'} or ref $opt->{'columns'} ne 'ARRAY') {
@@ -453,9 +432,7 @@ sub selectrow_array {
     my $self      = shift;
     my $statement = shift;
     my $opt       = shift;
-
-    # make opt hash keys lowercase
-    $opt = $self->_lowercase_and_verify_options($opt);
+    $opt          = {} unless defined $opt;
 
     my @result = @{$self->selectall_arrayref($statement, $opt, 1)};
     return @{$result[0]} if scalar @result > 0;
@@ -481,9 +458,7 @@ sub selectrow_arrayref {
     my $self      = shift;
     my $statement = shift;
     my $opt       = shift;
-
-    # make opt hash keys lowercase
-    $opt = $self->_lowercase_and_verify_options($opt);
+    $opt          = {} unless defined $opt;
 
     my $result = $self->selectall_arrayref($statement, $opt, 1);
     return if !defined $result;
@@ -511,9 +486,7 @@ sub selectrow_hashref {
     my $statement = shift;
     my $opt       = shift;
 
-    # make opt hash keys lowercase
-    $opt = $self->_lowercase_and_verify_options($opt);
-    $opt->{slice} = 1;
+    $opt->{'Slice'} = 1 unless defined $opt->{'Slice'};
 
     my $result = $self->selectall_arrayref($statement, $opt, 1);
     return if !defined $result;
@@ -541,9 +514,6 @@ sub selectscalar_value {
     my $statement = shift;
     my $opt       = shift;
 
-    # make opt hash keys lowercase
-    $opt = $self->_lowercase_and_verify_options($opt);
-
     my $row = $self->selectrow_arrayref($statement);
     return if !defined $row;
     return $row->[0] if scalar @{$row} > 0;
@@ -708,18 +678,10 @@ sub marked_bad {
 sub _send {
     my $self       = shift;
     my $statement  = shift;
-    my $opt        = shift;
-
-    delete $self->{'meta_data'};
-
+    my $with_peers = shift;
     my $header     = "";
     my $keys;
 
-    my $with_peers = 0;
-    if(defined $opt->{'addpeer'} and $opt->{'addpeer'}) {
-        $with_peers = 1;
-    }
-
     $Monitoring::Livestatus::ErrorCode = 0;
     undef $Monitoring::Livestatus::ErrorMessage;
 
@@ -776,11 +738,6 @@ sub _send {
 
     else {
 
-        # Add Limits header
-        if(defined $opt->{'limit_start'}) {
-            $statement .= "\nLimit: ".($opt->{'limit_start'} + $opt->{'limit_length'});
-        }
-
         # for querys with column header, no seperate columns will be returned
         if($statement =~ m/^Columns:\ (.*)$/mx) {
             ($statement,$keys) = $self->_extract_keys_from_columns_header($statement);
@@ -833,20 +790,9 @@ sub _send {
     my $peer_addr = $self->peer_addr;
     my $peer_key  = $self->peer_key;
 
-    my $limit_start = 0;
-    if(defined $opt->{'limit_start'}) { $limit_start = $opt->{'limit_start'}; }
-
     my @result;
-    my $row_count = 0;
     ## no critic
     for my $line (split/$line_seperator/m, $body) {
-        $row_count++;
-        next unless $row_count >= $limit_start;
-
-        if(defined $opt->{'limit_length'}) {
-            last if scalar @result >= $opt->{'limit_length'};
-        }
-
         my $row = [ split/$col_seperator/m, $line ];
         if(defined $with_peers and $with_peers == 1) {
             unshift @{$row}, $peer_name;
@@ -879,13 +825,7 @@ sub _send {
         unshift @{$keys}, 'peer_key';
     }
 
-    # set some metadata
-    $self->{'meta_data'} = {
-                    'row_count'    => $row_count,
-                    'result_count' => scalar @result,
-    };
-
-    return({ keys => $keys, result => \@result });
+    return({ keys => $keys, result => \@result});
 }
 
 ########################################
@@ -905,6 +845,9 @@ sub _open {
         $self->{'sock'} = $sock;
     }
 
+    # set timeout
+    $sock->timeout($self->{'timeout'}) if defined $sock;
+
     return($sock);
 }
 
@@ -954,44 +897,6 @@ useful when using multiple backends.
 
   see L<selectcol_arrayref> for more examples
 
-=head2 Deepcopy
-
-    deep copy/clone the result set.
-
-    Only effective when using multiple backends and threads.
-    This can be safely turned off if you dont change the
-    result set.
-    If you get an error like "Invalid value for shared scalar" error" this
-    should be turned on.
-
-    my $array_ref = $ml->selectcol_arrayref(
-       "GET hosts\nColumns: name contacts",
-       { Deepcopy => 1 }
-    );
-
-=head2 Limit
-
-    Just like the Limit: <nr> option from livestatus itself.
-    In addition you can add a start,length limit.
-
-    my $array_ref = $ml->selectcol_arrayref(
-       "GET hosts\nColumns: name contacts",
-       { Limit => "10,20" }
-    );
-
-    This example will return 20 rows starting at row 10. You will
-    get row 10-30.
-
-    Cannot be combined with a Limit inside the query
-    because a Limit will be added automatically.
-
-    Adding a limit this way will greatly increase performance and
-    reduce memory usage.
-
-    This option is multibackend safe contrary to the "Limit: " part of a statement.
-    Sending a statement like "GET...Limit: 10" with 3 backends will result in 30 rows.
-    Using this options, you will receive only the first 10 rows.
-
 =head2 Rename
 
   see L<COLUMN ALIAS> for detailed explainaton
@@ -1033,7 +938,7 @@ sub _send_socket {
         return('201', $self->_get_error(201), undef);
     }
 
-    $sock->read($header, 16) or return($self->_socket_error($statement, $sock, 'reading from socket failed: '.$!));
+    $sock->read($header, 16) or return($self->_socket_error($statement, $sock, 'reading header failed: '.$!));
     print "header: $header" if $self->{'verbose'};
     my($status, $msg, $content_length) = $self->_parse_header($header, $sock);
     return($status, $msg, undef) if !defined $content_length;
@@ -1055,6 +960,10 @@ sub _socket_error {
     my $message = "\n";
     $message   .= "peer                ".Dumper($self->peer_name);
     $message   .= "statement           ".Dumper($statement);
+    $message   .= "socket->sockname()  ".Dumper($sock->sockname());
+    $message   .= "socket->atmark()    ".Dumper($sock->atmark());
+    $message   .= "socket->error()     ".Dumper($sock->error());
+    $message   .= "socket->timeout()   ".Dumper($sock->timeout());
     $message   .= "message             ".Dumper($body);
 
     $self->{'logger'}->error($message) if defined $self->{'logger'};
@@ -1335,54 +1244,6 @@ sub _get_peers {
     return $peers;
 }
 
-
-########################################
-sub _lowercase_and_verify_options {
-    my $self   = shift;
-    my $opts   = shift;
-    my $return = {};
-
-    # list of allowed options
-    my $allowed_options = {
-        'addpeer'       => 1,
-        'backend'       => 1,
-        'columns'       => 1,
-        'deepcopy'      => 1,
-        'limit'         => 1,
-        'limit_start'   => 1,
-        'limit_length'  => 1,
-        'rename'        => 1,
-        'slice'         => 1,
-        'sum'           => 1,
-    };
-
-    for my $key (keys %{$opts}) {
-        if($self->{'warnings'} and !defined $allowed_options->{lc $key}) {
-            carp("unknown option used: $key - please use only: ".join(", ", keys %{$allowed_options}));
-        }
-        $return->{lc $key} = $opts->{$key};
-    }
-
-    # set limits
-    if(defined $return->{'limit'}) {
-        if(index($return->{'limit'}, ',') != -1) {
-            my($limit_start,$limit_length) = split /,/mx, $return->{'limit'};
-            $return->{'limit_start'}  = $limit_start;
-            $return->{'limit_length'} = $limit_length;
-        }
-        else {
-            $return->{'limit_start'}  = 0;
-            $return->{'limit_length'} = $return->{'limit'};
-        }
-        delete $return->{'limit'};
-    }
-
-    return($return);
-}
-
-
-########################################
-
 1;
 
 =head1 EXAMPLES
