@@ -6,8 +6,7 @@ use warnings;
 use Carp;
 use Data::Dumper;
 use Config;
-use Time::HiRes qw/gettimeofday tv_interval/;
-use Scalar::Util qw/looks_like_number/;
+use Time::HiRes qw( gettimeofday tv_interval );
 use Monitoring::Livestatus;
 use base "Monitoring::Livestatus";
 
@@ -113,16 +112,18 @@ See L<Monitoring::Livestatus> for more information.
 
 sub do {
     my $self  = shift;
-    my $opts  = $self->_lowercase_and_verify_options($_[1]);
+    my $opts  = $_[1];
     my $t0    = [gettimeofday];
 
-    $self->_do_on_peers("do", $opts->{'backends'}, @_);
+    # make opt hash keys lowercase
+    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
+
+    $self->_do_on_peers("do", $opts->{'backend'}, @_);
     my $elapsed = tv_interval ( $t0 );
     $self->{'logger'}->debug(sprintf('%.4f', $elapsed).' sec for do('.$_[0].') in total') if defined $self->{'logger'};
     return 1;
 }
 
-
 ########################################
 
 =head2 selectall_arrayref
@@ -133,17 +134,19 @@ See L<Monitoring::Livestatus> for more information.
 
 sub selectall_arrayref {
     my $self  = shift;
-    my $opts  = $self->_lowercase_and_verify_options($_[1]);
+    my $opts  = $_[1];
     my $t0    = [gettimeofday];
 
-    my $return  = $self->_merge_answer($self->_do_on_peers("selectall_arrayref", $opts->{'backends'}, @_));
+    # make opt hash keys lowercase
+    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
+
+    my $return  = $self->_merge_answer($self->_do_on_peers("selectall_arrayref", $opts->{'backend'}, @_));
     my $elapsed = tv_interval ( $t0 );
     $self->{'logger'}->debug(sprintf('%.4f', $elapsed).' sec for selectall_arrayref() in total') if defined $self->{'logger'};
 
     return $return;
 }
 
-
 ########################################
 
 =head2 selectall_hashref
@@ -154,17 +157,19 @@ See L<Monitoring::Livestatus> for more information.
 
 sub selectall_hashref {
     my $self  = shift;
-    my $opts  = $self->_lowercase_and_verify_options($_[2]);
+    my $opts  = $_[2];
     my $t0    = [gettimeofday];
 
-    my $return  = $self->_merge_answer($self->_do_on_peers("selectall_hashref", $opts->{'backends'}, @_));
+    # make opt hash keys lowercase
+    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
+
+    my $return  = $self->_merge_answer($self->_do_on_peers("selectall_hashref", $opts->{'backend'}, @_));
     my $elapsed = tv_interval ( $t0 );
     $self->{'logger'}->debug(sprintf('%.4f', $elapsed).' sec for selectall_hashref() in total') if defined $self->{'logger'};
 
     return $return;
 }
 
-
 ########################################
 
 =head2 selectcol_arrayref
@@ -175,17 +180,19 @@ See L<Monitoring::Livestatus> for more information.
 
 sub selectcol_arrayref {
     my $self  = shift;
-    my $opts  = $self->_lowercase_and_verify_options($_[1]);
+    my $opts  = $_[1];
     my $t0    = [gettimeofday];
 
-    my $return  = $self->_merge_answer($self->_do_on_peers("selectcol_arrayref", $opts->{'backends'}, @_));
+    # make opt hash keys lowercase
+    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
+
+    my $return  = $self->_merge_answer($self->_do_on_peers("selectcol_arrayref", $opts->{'backend'}, @_));
     my $elapsed = tv_interval ( $t0 );
     $self->{'logger'}->debug(sprintf('%.4f', $elapsed).' sec for selectcol_arrayref() in total') if defined $self->{'logger'};
 
     return $return;
 }
 
-
 ########################################
 
 =head2 selectrow_array
@@ -197,17 +204,20 @@ See L<Monitoring::Livestatus> for more information.
 sub selectrow_array {
     my $self      = shift;
     my $statement = $_[0];
-    my $opts      = $self->_lowercase_and_verify_options($_[1]);
+    my $opts      = $_[1];
     my $t0        = [gettimeofday];
     my @return;
 
-    if((defined $opts->{'sum'} and $opts->{'sum'} == 1) or (!defined $opts->{'sum'} and $statement =~ m/^Stats:/mx)) {
-        @return = @{$self->_sum_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backends'}, @_))};
+    # make opt hash keys lowercase
+    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
+
+    if(defined $opts->{'sum'} or $statement =~ m/^Stats:/mx) {
+        @return = @{$self->_sum_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backend'}, @_))};
     } else {
         if($self->{'warnings'}) {
             carp("selectrow_arrayref without Stats on multi backend will not work as expected!");
         }
-        my $rows = $self->_merge_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backends'}, @_));
+        my $rows = $self->_merge_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backend'}, @_));
         @return = @{$rows} if defined $rows;
     }
 
@@ -217,7 +227,6 @@ sub selectrow_array {
     return @return;
 }
 
-
 ########################################
 
 =head2 selectrow_arrayref
@@ -229,17 +238,20 @@ See L<Monitoring::Livestatus> for more information.
 sub selectrow_arrayref {
     my $self      = shift;
     my $statement = $_[0];
-    my $opts      = $self->_lowercase_and_verify_options($_[1]);
+    my $opts      = $_[1];
     my $t0        = [gettimeofday];
     my $return;
 
-    if((defined $opts->{'sum'} and $opts->{'sum'} == 1) or (!defined $opts->{'sum'} and $statement =~ m/^Stats:/mx)) {
-        $return = $self->_sum_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backends'}, @_));
+    # make opt hash keys lowercase
+    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
+
+    if(defined $opts->{'sum'} or $statement =~ m/^Stats:/mx) {
+        $return = $self->_sum_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backend'}, @_));
     } else {
         if($self->{'warnings'}) {
             carp("selectrow_arrayref without Stats on multi backend will not work as expected!");
         }
-        my $rows = $self->_merge_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backends'}, @_));
+        my $rows = $self->_merge_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backend'}, @_));
         $return = $rows->[0] if defined $rows->[0];
     }
 
@@ -249,7 +261,6 @@ sub selectrow_arrayref {
     return $return;
 }
 
-
 ########################################
 
 =head2 selectrow_hashref
@@ -261,19 +272,22 @@ See L<Monitoring::Livestatus> for more information.
 sub selectrow_hashref {
     my $self      = shift;
     my $statement = $_[0];
-    my $opts      = $self->_lowercase_and_verify_options($_[1]);
+    my $opts      = $_[1];
 
     my $t0 = [gettimeofday];
 
     my $return;
 
-    if((defined $opts->{'sum'} and $opts->{'sum'} == 1) or (!defined $opts->{'sum'} and $statement =~ m/^Stats:/mx)) {
-        $return = $self->_sum_answer($self->_do_on_peers("selectrow_hashref", $opts->{'backends'}, @_));
+    # make opt hash keys lowercase
+    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
+
+    if(defined $opts->{'sum'} or $statement =~ m/^Stats:/mx) {
+        $return = $self->_sum_answer($self->_do_on_peers("selectrow_hashref", $opts->{'backend'}, @_));
     } else {
         if($self->{'warnings'}) {
             carp("selectrow_hashref without Stats on multi backend will not work as expected!");
         }
-        $return = $self->_merge_answer($self->_do_on_peers("selectrow_hashref", $opts->{'backends'}, @_));
+        $return = $self->_merge_answer($self->_do_on_peers("selectrow_hashref", $opts->{'backend'}, @_));
     }
 
     my $elapsed = tv_interval ( $t0 );
@@ -282,7 +296,6 @@ sub selectrow_hashref {
     return $return;
 }
 
-
 ########################################
 
 =head2 selectscalar_value
@@ -294,19 +307,22 @@ See L<Monitoring::Livestatus> for more information.
 sub selectscalar_value {
     my $self  = shift;
     my $statement = $_[0];
-    my $opts      = $self->_lowercase_and_verify_options($_[1]);
+    my $opts      = $_[1];
 
     my $t0 = [gettimeofday];
 
+    # make opt hash keys lowercase
+    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
+
     my $return;
 
-    if((defined $opts->{'sum'} and $opts->{'sum'} == 1) or (!defined $opts->{'sum'} and $statement =~ m/^Stats:/mx)) {
-        return $self->_sum_answer($self->_do_on_peers("selectscalar_value", $opts->{'backends'}, @_));
+    if(defined $opts->{'sum'} or $statement =~ m/^Stats:/mx) {
+        return $self->_sum_answer($self->_do_on_peers("selectscalar_value", $opts->{'backend'}, @_));
     } else {
         if($self->{'warnings'}) {
             carp("selectscalar_value without Stats on multi backend will not work as expected!");
         }
-        my $rows = $self->_merge_answer($self->_do_on_peers("selectscalar_value", $opts->{'backends'}, @_));
+        my $rows = $self->_merge_answer($self->_do_on_peers("selectscalar_value", $opts->{'backend'}, @_));
 
         $return = $rows->[0] if defined $rows->[0];
     }
@@ -317,7 +333,6 @@ sub selectscalar_value {
     return $return;
 }
 
-
 ########################################
 
 =head2 errors_are_fatal
@@ -332,7 +347,6 @@ sub errors_are_fatal {
     return $self->_change_setting('errors_are_fatal', $value);
 }
 
-
 ########################################
 
 =head2 warnings
@@ -347,7 +361,6 @@ sub warnings {
     return $self->_change_setting('warnings', $value);
 }
 
-
 ########################################
 
 =head2 verbose
@@ -450,7 +463,6 @@ sub _change_setting {
     return $old;
 }
 
-
 ########################################
 sub _start_worker {
     my $self = shift;
@@ -476,7 +488,6 @@ sub _start_worker {
     return;
 }
 
-
 ########################################
 sub _stop_worker {
     # try to kill our threads safely
@@ -488,7 +499,6 @@ sub _stop_worker {
     return;
 }
 
-
 ########################################
 sub _worker_thread {
     local $SIG{'USR1'} = sub { threads->exit(); };
@@ -512,7 +522,6 @@ sub _worker_thread {
     return;
 }
 
-
 ########################################
 sub _do_wrapper {
     my $peer   = shift;
@@ -537,26 +546,20 @@ sub _do_wrapper {
     return $return;
 }
 
-
 ########################################
 sub _do_on_peers {
-    my $self        = shift;
-    my $sub         = shift;
-    my $backends    = shift;
-    my @opts        = @_;
-    my $statement   = $opts[0];
-    my $use_threads = $self->{'use_threads'};
-    my $t0          = [gettimeofday];
+    my $self      = shift;
+    my $sub       = shift;
+    my $backends  = shift;
+    my @opts      = @_;
+    my $statement = $opts[0];
+
+    my $t0 = [gettimeofday];
 
     my $return;
     my %codes;
     my %messages;
-    my $query_options;
-    if($sub eq 'selectall_hashref') {
-        $query_options = $self->_lowercase_and_verify_options($opts[2]);
-    } else {
-        $query_options = $self->_lowercase_and_verify_options($opts[1]);
-    }
+    my $use_threads = $self->{'use_threads'};
 
     # which peers affected?
     my @peers;
@@ -582,27 +585,18 @@ sub _do_on_peers {
     # its faster without threads for only one peer
     if(scalar @peers <= 1) { $use_threads = 0; }
 
-    # if we have limits set, we cannot use threads
-    if(defined $query_options->{'limit_start'}) { $use_threads = 0; }
-
     if($use_threads) {
         # use the threaded variant
         print("using threads\n") if $self->{'verbose'};
 
-        my $peers_to_use;
-        for my $peer (@peers) {
-            $peers_to_use->{$peer->peer_key} = 1;
-        }
         my $x = 0;
-        for my $peer (@{$self->{'peers'}}) {
-            if(defined $peers_to_use->{$peer->peer_key}) {
-                my $job = {
-                        'peer'   => $x,
-                        'sub'    => $sub,
-                        'opts'   => \@opts,
-                };
-                $self->{'WorkQueue'}->enqueue($job);
-            }
+        for my $peer (@peers) {
+            my $job = {
+                    'peer'   => $x,
+                    'sub'    => $sub,
+                    'opts'   => \@opts,
+            };
+            $self->{'WorkQueue'}->enqueue($job);
             $x++;
         }
 
@@ -625,15 +619,6 @@ sub _do_on_peers {
                 my $erg = _do_wrapper($peer, $sub, $self->{'logger'}, @opts);
                 $return->{$peer->peer_key} = $erg->{'data'};
                 push @{$codes{$erg->{'code'}}}, { 'peer' => $peer, 'msg' => $erg->{'msg'} };
-
-                # compute limits
-                if(defined $query_options->{'limit_length'} and $peer->{'meta_data'}->{'result_count'}) {
-                    last;
-                }
-                # set a new start if we had rows already
-                if(defined $query_options->{'limit_start'}) {
-                    $query_options->{'limit_start'} = $query_options->{'limit_start'} - $peer->{'meta_data'}->{'row_count'};
-                }
             }
         }
     }
@@ -642,51 +627,36 @@ sub _do_on_peers {
     # check if we different result stati
     undef $Monitoring::Livestatus::ErrorMessage;
     $Monitoring::Livestatus::ErrorCode = 0;
-    my @codes = sort keys %codes;
+    my @codes = keys %codes;
     if(scalar @codes > 1) {
         # got different results for our backends
         print "got different result stati: ".Dumper(\%codes) if $self->{'verbose'};
     } else {
         # got same result codes for all backend
-    }
-
-    my $failed = 0;
-    my $code = $codes[0];
-    if($code >= 300) {
-        $failed = 1;
-    }
-
-    if($failed) {
-        my $msg  = $codes{$code}->[0]->{'msg'};
-        print "same: $code -> $msg\n" if $self->{'verbose'};
-        $Monitoring::Livestatus::ErrorMessage = $msg;
-        $Monitoring::Livestatus::ErrorCode    = $code;
-        if($self->{'errors_are_fatal'}) {
-            croak("ERROR ".$code." - ".$Monitoring::Livestatus::ErrorMessage." in query:\n'".$statement."'\n");
+        my $code = $codes[0];
+        if($code >= 300) {
+            my $msg  = $codes{$code}->[0]->{'msg'};
+            print "same: $code -> $msg\n" if $self->{'verbose'};
+            $Monitoring::Livestatus::ErrorMessage = $msg;
+            $Monitoring::Livestatus::ErrorCode    = $code;
+            if($self->{'errors_are_fatal'}) {
+                croak("ERROR ".$code." - ".$Monitoring::Livestatus::ErrorMessage." in query:\n'".$statement."'\n");
+            }
+            return;
         }
-        return;
     }
 
     my $elapsed = tv_interval ( $t0 );
     $self->{'logger'}->debug(sprintf('%.4f', $elapsed).' sec for fetching all data') if defined $self->{'logger'};
 
-    # deep copy result?
-    if($use_threads
-       and (
-            (defined $query_options->{'deepcopy'} and $query_options->{'deepcopy'} == 1)
-            or
-            (defined $self->{'deepcopy'}          and $self->{'deepcopy'} == 1)
-        )
-       ) {
+    if($use_threads) {
         # result has to be cloned to avoid "Invalid value for shared scalar" error
-
         $return = $self->_clone($return, $self->{'logger'});
     }
 
     return($return);
 }
 
-
 ########################################
 sub _merge_answer {
     my $self   = shift;
@@ -717,7 +687,6 @@ sub _merge_answer {
     return($return);
 }
 
-
 ########################################
 sub _sum_answer {
     my $self   = shift;
@@ -729,7 +698,7 @@ sub _sum_answer {
             for my $key (keys %{$data->{$peername}}) {
                 if(!defined $return->{$key}) {
                     $return->{$key} = $data->{$peername}->{$key};
-                } elsif(looks_like_number($data->{$peername}->{$key})) {
+                } else {
                     $return->{$key} += $data->{$peername}->{$key};
                 }
             }
@@ -757,7 +726,6 @@ sub _sum_answer {
     return $return;
 }
 
-
 ########################################
 sub _clone {
     my $self   = shift;
@@ -791,7 +759,6 @@ sub _clone {
     return $return;
 }
 
-
 ########################################
 sub _get_peer_by_key {
     my $self = shift;
@@ -803,7 +770,6 @@ sub _get_peer_by_key {
     return $self->{'peer_by_key'}->{$key};
 }
 
-
 ########################################
 
 END {
